@@ -1,9 +1,3 @@
-# Copyright (c) 2000-2003 Dave Rolsky
-# All rights reserved.
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.  See the LICENSE
-# file that comes with this distribution for more details.
-
 package Params::Validate;
 
 use strict;
@@ -11,11 +5,12 @@ use strict;
 BEGIN
 {
     use Exporter;
-    use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS %OPTIONS $options );
+    use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK
+                 %EXPORT_TAGS %OPTIONS $options $NO_VALIDATION );
 
     @ISA = 'Exporter';
 
-    $VERSION = '0.58';
+    $VERSION = '0.89';
 
     my %tags =
         ( types =>
@@ -32,16 +27,17 @@ BEGIN
     @EXPORT_OK = ( @{ $EXPORT_TAGS{all} }, 'set_options' );
     @EXPORT = qw( validate validate_pos );
 
-    eval { require Params::ValidateXS } unless $ENV{PV_TEST_PERL};
+    $NO_VALIDATION = $ENV{PERL_NO_VALIDATION};
+
+    eval { require Params::ValidateXS; } unless $ENV{PV_TEST_PERL};
 
     if ( $@ || $ENV{PV_TEST_PERL} )
     {
-        # suppress a subroutine redefined warning
-        undef &Params::Validate::validation_options;
         require Params::ValidatePP;
     }
 }
 
+
 1;
 
 __END__
@@ -105,7 +101,7 @@ Params::Validate - Validate method/function parameters
 
   sub pos_with_defaults
   {
-       my @p = validate( @_, 1, { default => 99 } );
+       my @p = validate_pos( @_, 1, { default => 99 } );
   }
 
   sub sets_options_on_call
@@ -113,8 +109,7 @@ Params::Validate - Validate method/function parameters
        my %p = validate_with
                    ( params => \@_,
                      spec   => { foo => { type SCALAR, default => 2 } },
-                     ignore_case   => 1,
-                     strip_leading => '-',
+                     normalize_keys => sub { $_[0] =~ s/^-//; lc $_[0] },
                    );
   }
 
@@ -132,7 +127,7 @@ arguments.
 
 =head2 EXPORT
 
-The module always exports the C<validate> and C<validate_pos>
+The module always exports the C<validate()> and C<validate_pos()>
 functions.
 
 It also has an additional function available for export,
@@ -147,7 +142,7 @@ in the section on L<Type Validation|Params::Validate/Type Validation>.
 
 The constants are available via the export tag C<:types>.  There is
 also an C<:all> tag which includes all of the constants as well as the
-C<validation_options> function.
+C<validation_options()> function.
 
 =head1 PARAMETER VALIDATION
 
@@ -158,10 +153,10 @@ validation specification is given to the relevant subroutine.  The
 other difference is in the error messages produced when validation
 checks fail.
 
-When handling named parameters, the module is capable of handling
-either a hash or a hash reference transparently.
+When handling named parameters, the module will accept either a hash
+or a hash reference.
 
-Subroutines expecting named parameters should call the C<validate>
+Subroutines expecting named parameters should call the C<validate()>
 subroutine like this:
 
  validate( @_, { parameter1 => validation spec,
@@ -170,7 +165,7 @@ subroutine like this:
                } );
 
 Subroutines expecting positional parameters should call the
-C<validate_pos> subroutine like this:
+C<validate_pos()> subroutine like this:
 
  validate_pos( @_, { validation spec }, { validation spec } );
 
@@ -183,8 +178,8 @@ For a subroutine expecting named parameters, you would do this:
 
  validate( @_, { foo => 1, bar => 1, baz => 0 } );
 
-This says that the C<foo> and C<bar> parameters are mandatory and that
-the C<baz> parameter is optional.  The presence of any other
+This says that the "foo" and "bar" parameters are mandatory and that
+the "baz" parameter is optional.  The presence of any other
 parameters will cause an error.
 
 For a subroutine expecting positional parameters, you would do this:
@@ -214,7 +209,7 @@ it is considered optional.
 =head2 Type Validation
 
 This module supports the following simple types, which can be
-L<exported as constants|EXPORT>:
+L<exported as constants|/EXPORT>:
 
 =over 4
 
@@ -231,7 +226,7 @@ An array reference such as C<[1, 2, 3]> or C<\@foo>.
 
 =item * HASHREF
 
-A hash reference such as C<{ a => 1, b => 2 }> or C<\%bar>.
+A hash reference such as C<< { a => 1, b => 2 } >> or C<\%bar>.
 
 =item * CODEREF
 
@@ -319,8 +314,8 @@ probably better to specify what methods you expect an object to
 have rather than what class it should be of (or a child of).  This
 will make your API much more flexible.
 
-With that said, if you want to validate_with that an incoming
-parameter belongs to a class (or child class) or classes, do:
+With that said, if you want to validate that an incoming parameter
+belongs to a class (or child class) or classes, do:
 
  validate( @_,
            { foo =>
@@ -347,6 +342,10 @@ example:
 The value of the "regex" key may be either a string or a pre-compiled
 regex created via C<qr>.
 
+If the value being checked against a regex is undefined, the regex is
+explicitly checked against the empty string ('') instead, in order to
+avoid "Use of uninitialized value" warnings.
+
 The C<Regexp::Common> module on CPAN is an excellent source of regular
 expressions suitable for validating input.
 
@@ -354,21 +353,43 @@ expressions suitable for validating input.
 
 If none of the above are enough, it is possible to pass in one or more
 callbacks to validate the parameter.  The callback will be given the
-B<value> of the parameter as its sole argument.  Callbacks are
-specified as hash reference.  The key is an id for the callback (used
-in error messages) and the value is a subroutine reference, such as:
+B<value> of the parameter as its first argument.  Its second argument
+will be all the parameters, as a reference to either a hash or array.
+Callbacks are specified as hash reference.  The key is an id for the
+callback (used in error messages) and the value is a subroutine
+reference, such as:
 
  validate( @_,
            { foo =>
-             callbacks =>
-             { 'smaller than a breadbox' => sub { shift() < $breadbox },
-               'green or blue' =>
-                sub { $_[0] eq 'green' || $_[0] eq 'blue' } } } );
+             { callbacks =>
+               { 'smaller than a breadbox' => sub { shift() < $breadbox },
+                 'green or blue' =>
+                  sub { $_[0] eq 'green' || $_[0] eq 'blue' } } } );
 
-On a side note, I would highly recommend taking a look at Damian
-Conway's Regexp::Common module, which could greatly simply the
-callbacks you use, as it provides patterns useful for validating all
-sorts of data.
+ validate( @_,
+           { foo =>
+             { callbacks =>
+               { 'bigger than baz' => sub { $_[0] > $_[1]->{baz} } } } } );
+
+=head2 Untainting
+
+If you want values untainted, set the "untaint" key in a spec hashref
+to a true value, like this:
+
+ my %p =
+   validate( @_, { foo =>
+                   { type => SCALAR, untaint => 1 },
+                   bar =>
+                   { type => ARRAYREF } } );
+
+This will untaint the "foo" parameter if the parameters are valid.
+
+Note that untainting is only done if I<all parameters> are valid.
+Also, only the return values are untainted, not the original values
+passed into the validation function.
+
+Asking for untainting of a reference value will not do anything, as
+C<Params::Validate> will only attempt to untaint the reference itself.
 
 =head2 Mandatory/Optional Revisited
 
@@ -387,11 +408,44 @@ or this for positional parameters:
 By default, parameters are assumed to be mandatory unless specified as
 optional.
 
+=head2 Dependencies
+
+It also possible to specify that a given optional parameter depends on
+the presence of one or more other optional parameters.
+
+ validate( @_, { cc_number =>
+                 { type => SCALAR, optional => 1,
+                   depends => [ 'cc_expiration', 'cc_holder_name' ],
+                 },
+                 cc_expiration
+                 { type => SCALAR, optional => 1 },
+                 cc_holder_name
+                 { type => SCALAR, optional => 1 },
+               } );
+
+In this case, "cc_number", "cc_expiration", and "cc_holder_name" are
+all optional.  However, if "cc_number" is provided, then
+"cc_expiration" and "cc_holder_name" must be provided as well.
+
+This allows you to group together sets of parameters that all must be
+provided together.
+
+The C<validate_pos()> version of dependencies is slightly different,
+in that you can only depend on one other parameter.  Also, if for
+example, the second parameter 2 depends on the fourth parameter, then
+it implies a dependency on the third parameter as well.  This is
+because if the fourth parameter is required, then the user must also
+provide a third parameter so that there can be four parameters in
+total.
+
+C<Params::Validate> will die if you try to depend on a parameter not
+declared as part of your parameter specification.
+
 =head2 Specifying defaults
 
-If the C<validate> or C<validate_pos> functions are called in a list
-context, they will return an array or hash containing the original
-parameters plus defaults as indicated by the validation spec.
+If the C<validate()> or C<validate_pos()> functions are called in a
+list context, they will return an array or hash containing the
+original parameters plus defaults as indicated by the validation spec.
 
 If the function is not called in a list context, providing a default
 in the validation spec still indicates that the parameter is optional.
@@ -414,7 +468,7 @@ returned, as appropriate.
 =head2 Validation failure
 
 By default, when validation fails C<Params::Validate> calls
-C<Carp::confess>.  This can be overridden by setting the C<on_fail>
+C<Carp::confess()>.  This can be overridden by setting the C<on_fail>
 option, which is described in the L<"GLOBAL" OPTIONS|"GLOBAL" OPTIONS>
 section.
 
@@ -422,13 +476,13 @@ section.
 
 When using this module to validate the parameters passed to a method
 call, you will probably want to remove the class/object from the
-parameter list B<before> calling C<validate> or C<validate_pos>.  If
-your method expects named parameters, then this is necessary for the
-C<validate> function to actually work, otherwise C<@_> will not
-contain a hash, but rather your object (or class) B<followed> by a
-hash.
+parameter list B<before> calling C<validate()> or C<validate_pos()>.
+If your method expects named parameters, then this is necessary for
+the C<validate()> function to actually work, otherwise C<@_> will not
+be useable as a hash, because it will first have your object (or
+class) B<followed> by a set of keys and values.
 
-Thus the idiomatic usage of C<validate> in a method call will look
+Thus the idiomatic usage of C<validate()> in a method call will look
 something like this:
 
  sub method
@@ -440,9 +494,9 @@ something like this:
 
 =head1 "GLOBAL" OPTIONS
 
-Because the calling syntax for the C<validate> and C<validate_pos>
-functions does not make it possible to specify any options other than
-the the validation spec, it is possible to set some options as
+Because the API for the C<validate()> and C<validate_pos()> functions
+does not make it possible to specify any options other than the the
+validation spec, it is possible to set some options as
 pseudo-'globals'.  These allow you to specify such things as whether
 or not the validation of named parameters should be case sensitive,
 for one example.
@@ -452,8 +506,8 @@ B<only applied to calls originating from the package that set the
 options>.
 
 In other words, if I am in package C<Foo> and I call
-C<Params::Validate::validation_options>, those options are only in
-effect when I call C<validate> from package C<Foo>.
+C<validation_options()>, those options are only in effect when I call
+C<validate()> from package C<Foo>.
 
 While this is quite different from how most other modules operate, I
 feel that this is necessary in able to make it possible for one
@@ -463,31 +517,48 @@ options set.
 
 The downside to this is that if you are writing an app with a standard
 calling style for all functions, and your app has ten modules, B<each
-module must include a call to C<Params::Validate::validation_options>>.
+module must include a call to C<validation_options()>>. You could of
+course write a module that all your modules use which uses various
+trickery to do this when imported.
 
 =head2 Options
 
 =over 4
 
-=item * ignore_case => $boolean
+=item * normalize_keys => $callback
 
-This is only relevant when dealing with named parameters.  If it is
-true, then the validation code will ignore the case of parameter
-names.  Defaults to false.
+This option is only relevant when dealing with named parameters.
 
-When this is turned on, we have to copy more data around internally,
-leading to a potential speed impact.
+This callback will be used to transform the hash keys of both the
+parameters and the parameter spec when C<validate()> or
+C<validate_with()> are called.
 
-=item * strip_leading => $characters
+Any alterations made by this callback will be reflected in the
+parameter hash that is returned by the validation function.  For
+example:
 
-This too is only relevant when dealing with named parameters.  If this
-is given then any parameters starting with these characters will be
-considered equivalent to parameters without them entirely.  For
-example, if this is specified as '-', then C<-foo> and C<foo> would be
-considered identical.
+  sub foo {
+      return
+        validate_with( params => \@_,
+                       spec   => { foo => { type => SCALAR } },
+                       normalize_keys =>
+                       sub { my $k = shift; $k =~ s/^-//; return uc $k },
+                     );
 
-When this is turned on, we have to copy more data around internally,
-leading to a potential speed impact.
+  }
+
+  %p = foo( foo => 20 );
+
+  # $p{FOO} is now 20
+
+  %p = foo( -fOo => 50 );
+
+  # $p{FOO} is now 50
+
+The callback must return a defined value.
+
+If a callback is given than the deprecated "ignore_case" and
+"strip_leading" options are ignored.
 
 =item * allow_extra => $boolean
 
@@ -504,7 +575,7 @@ string describing the failure.  This is useful if you wish to have
 this module throw exceptions as objects rather than as strings, for
 example.
 
-This callback is expected to C<die> internally.  If it does not, the
+This callback is expected to C<die()> internally.  If it does not, the
 validation will proceed onwards, with unpredictable results.
 
 The default is to simply use the Carp module's C<confess()> function.
@@ -513,16 +584,34 @@ The default is to simply use the Carp module's C<confess()> function.
 
 This tells Params::Validate how many stack frames to skip when finding
 a subroutine name to use in error messages.  By default, it looks one
-frame back, at the immediate caller to C<validate> or C<validate_pos>.
-If this option is set, then the given number of frames are skipped
-instead.
+frame back, at the immediate caller to C<validate()> or
+C<validate_pos()>.  If this option is set, then the given number of
+frames are skipped instead.
+
+=item * ignore_case => $boolean
+
+DEPRECATED
+
+This is only relevant when dealing with named parameters.  If it is
+true, then the validation code will ignore the case of parameter
+names.  Defaults to false.
+
+=item * strip_leading => $characters
+
+DEPRECATED
+
+This too is only relevant when dealing with named parameters.  If this
+is given then any parameters starting with these characters will be
+considered equivalent to parameters without them entirely.  For
+example, if this is specified as '-', then C<-foo> and C<foo> would be
+considered identical.
 
 =back
 
 =head1 PER-INVOCATION OPTIONS
 
-The C<validate_with> function can be used to set the options listed above on
-a per-invocation basis.  For example:
+The C<validate_with()> function can be used to set the options listed
+above on a per-invocation basis.  For example:
 
   my %p =
       validate_with
@@ -533,13 +622,13 @@ a per-invocation basis.  For example:
           );
 
 In addition to the options listed above, it is also possible to set
-the option C<called>, which should be a string.  This string will be
+the option "called", which should be a string.  This string will be
 used in any error messages caused by a failure to meet the validation
 spec.
 
-This subroutine will validate named parameters as a hash if the
-C<spec> parameter is a hash reference.  If it is an array reference,
-the parameters are assumed to be positional.
+This subroutine will validate named parameters as a hash if the "spec"
+parameter is a hash reference.  If it is an array reference, the
+parameters are assumed to be positional.
 
   my %p =
       validate_with
@@ -562,15 +651,37 @@ the parameters are assumed to be positional.
 =head1 DISABLING VALIDATION
 
 If the environment variable C<PERL_NO_VALIDATION> is set to something
-true, then all calls to the validation functions are turned into
-no-ops.  This may be useful if you only want to use this module during
-development but don't want the speed hit during production.
+true, then validation is turned off.  This may be useful if you only
+want to use this module during development but don't want the speed
+hit during production.
 
 The only error that will be caught will be when an odd number of
 parameters are passed into a function/method that expects a hash.
 
-This environment value is checked B<only> when the module is first
-loaded.  You cannot change it after the module has loaded.
+If you want to selectively turn validation on and off at runtime, you
+can directly set the C<$Params::Validate::NO_VALIDATION> global
+variable.  It is B<strongly> recommended that you B<localize> any
+changes to this variable, because other modules you are using may
+expect validation to be on when they execute.  For example:
+
+
+  {
+      local $Params::Validate::NO_VALIDATION = 1;
+      # no error
+      foo( bar => 2 );
+  }
+
+  # error
+  foo( bar => 2 );
+
+  sub foo
+  {
+      my %p = validate( @_, { foo => 1 } );
+      ...
+  }
+
+But if you want to shoot yourself in the foot and just turn it off, go
+ahead!
 
 =head1 LIMITATIONS
 
@@ -582,18 +693,24 @@ future.
 Ideally, there would be only one validation function.  If someone
 figures out how to do this, please let me know.
 
-=head1 SEE ALSO
+=head1 SUPPORT
 
-Getargs::Long - similar capabilities with a different interface.  If
-you like what Params::Validate does but not its 'feel' try this one
-instead.
+Please submit bugs and patches to the CPAN RT system at
+http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Params%3A%3AValidate or
+via email at bug-params-validate@rt.cpan.org.
 
-Carp::Assert and Class::Contract - other modules in the general spirit
-of validating that certain things are true before/while/after
-executing actual program code.
+Support questions can be sent to Dave at autarch@urth.org.
+
+The code repository is at https://svn.urth.org/svn/Params-Validate/
 
 =head1 AUTHORS
 
 Dave Rolsky, <autarch@urth.org> and Ilya Martynov <ilya@martynov.org>
 
+=head1 COPYRIGHT
+
+Copyright (c) 2004-2007 David Rolsky.  All rights reserved.  This
+program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
+
 =cut
